require 'ffi'
require 'flagger/entity'
require 'flagger/response'
require 'flagger/native'
require 'flagger/version'

module Flagger
  include FlaggerNative, ResponseParser

  # init method gets FlaggerConfiguration, establishes and maintains SSE connections and initializes Ingester
  # init must be called only once, at the start of your application.
  # Your program must wait for init to finish before using any other Flagger methods
  def Flagger.init(*args)
      case args.size
        when 1
            request = {
                apiKey: args[0][:api_key],
                sdkName: "ruby",
                sdkVersion: Flagger::VERSION,
                sourceURL: args[0][:source_url],
                backupSourceURL: args[0][:backup_source_url],
                sseURL: args[0][:sse_url],
                ingestionURL: args[0][:ingestion_url],
                logLevel: args[0][:log_level],
            }.compact.to_json
            ResponseParser::parse_error(FlaggerNative::Init(request))
        when 2
            request = {
                    apiKey: args[0],
                    sdkName: "ruby",
                    sdkVersion: Flagger::VERSION,
                    sourceURL: args[1][:source_url],
                    backupSourceURL: args[1][:backup_source_url],
                    sseURL: args[1][:sse_url],
                    ingestionURL: args[1][:ingestion_url],
                    logLevel: args[1][:log_level],
                }.compact.to_json
            ResponseParser::parse_error(FlaggerNative::Init(request))
      else
        raise "Invalid number of arguments, expected 1 or 2"
      end
  end

  # Explicitly notify Airship about an Entity
  def Flagger.publish(entity)
    request = "{\"entity\": #{entity.to_json} }"
    ResponseParser::parse_error(FlaggerNative::Publish(request))
  end

  # Simple event tracking API. Entity is an optional parameter if it was set before.
  def Flagger.track(name, event_properties, *entity)
    request = entity.empty? ? "{\"event\":{\"name\":\"#{name}\", \"eventProperties\": #{event_properties.to_json}}"
                  : "{\"event\":{\"name\":\"#{name}\", \"eventProperties\": #{event_properties.to_json}, \"entity\":#{entity[0].to_json}}}"
    ResponseParser::parse_error(FlaggerNative::Track(request))
  end

  # set_entity stores an entity in Flagger, which allows omission of entity in other API methods.
  #
  # If you don't provide any entity to Flagger:
  # - flag functions always resolve with the default variation
  # - track method doesn't record an event
  def Flagger.set_entity(entity)
    ResponseParser::parse_error(FlaggerNative::SetEntity("{\"entity\":#{entity.to_json}}"))
  end

  # Determines if flag is enabled for entity.
  def Flagger.is_enabled(codename, *entity)
    request = entity.empty? ? "{\"codename\": \"#{codename}\"}" : "{\"codename\": \"#{codename}\", \"entity\":#{entity[0].to_json}}"
    ResponseParser::parse_error FlaggerNative::FlagIsEnabled(request)
  end

  # Determines if entity is within the targeted subpopulations
  def Flagger.is_sampled(codename, *entity)
    request = entity.empty? ? "{\"codename\": \"#{codename}\"}" : "{\"codename\": \"#{codename}\", \"entity\":#{entity[0].to_json}}"
    ResponseParser::parse_error FlaggerNative::FlagIsSampled(request)
  end

  # Returns the variation assigned to the entity in a multivariate flag
  def Flagger.get_variation(codename, *entity)
    request = entity.empty? ? "{\"codename\": \"#{codename}\"}" : "{\"codename\": \"#{codename}\", \"entity\":#{entity[0].to_json}}"
    ResponseParser::parse_error FlaggerNative::FlagGetVariation(request)
  end

  # Returns the payload associated with the treatment assigned to the entity
  def Flagger.get_payload(codename, *entity)
    request = entity.empty? ? "{\"codename\": \"#{codename}\"}" : "{\"codename\": \"#{codename}\", \"entity\":#{entity[0].to_json}}"
    ResponseParser::parse_error FlaggerNative::FlagGetPayload(request)
  end

  # shutdown ingests data(if any), stop ingester and closes SSE connection. shutdown waits to finish current ingestion
  # request, but no longer than a timeoutMillis.
  # returns true if closed by timeout
  def Flagger.shutdown(timeout)
    request = "{\"timeout\": #{timeout}}"
    ResponseParser::parse_error FlaggerNative::Shutdown(request)
  end

end

